// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#pragma once

#include <cstdint>

namespace spex {

// OGSE/EGSE settings defined under the gse_data group for
// polarimetric calibration measurements
struct EGSE
{
    double ITOS_time {};
    double NOMHK_packets_time {};
    uint8_t LDLS_STATUS {};
    uint8_t POLARIZER_MOVING {};
    uint8_t SHUTTER_STAGE_MOVING {};
    uint8_t STST_POLARIZER_MOVING {};
    uint8_t ALT_STAGE_MOVING {};
    uint8_t ACT_STAGE_MOVING {};
    uint8_t GP_0_MOVING {};
    uint8_t GP_1_MOVING {};
    uint8_t C_FLEX_405nm {};
    uint8_t C_FLEX_457nm {};
    uint8_t C_FLEX_515nm {};
    uint8_t C_FLEX_561nm {};
    uint8_t C_FLEX_660nm {};
    uint8_t COBOLT_785nm {};
    uint8_t CRYSTA_STATUS {};
    // 3 bytes for alignment
    uint32_t HK_TS1 {};
    uint32_t HK_TS2 {};
    float SURV_TST1 {};
    float SURV_TST2 {};
    float AI_02 {};
    float AI_03 {};
    float AI_04 {};
    float AI_05 {};
    float AI_06 {};
    float AI_07 {};
    float AI_08 {};
    float AI_09 {};
    float V_OUT_ICU {};
    float I_OUT_ICU {};
    float V_OUT_HTR {};
    float I_OUT_HTR {};
    float POLARIZER {};
    float SHUTTER_STAGE {};
    float STST_POLARIZER {};
    float ALT_ANGLE {};
    float ACT_ANGLE {};
    float GP_0_ANGLE {};
    float GP_1_ANGLE {};
};

// For identifying which EGSE member to use in certain contexts. This
// is typically set by user input.
enum class EGSEMember
{
    POLARIZER,
    ACT_ANGLE,
};

} // namespace spex
