// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#pragma once

#include <vector>

namespace spex {

int juliandate(
    int day,
    int month,
    int year
);

int chol_solve(
    std::size_t nstate, // Size of the matrix (square)
    double *mat // Matrix (will be turned into her inverse. It is represented as just one triangle, because the matrix must be symmetric for this algorithm.
);

// Linear interpolation (not using B-splines)
// The independent values can be ascending or descending per array
void linear_interpol(
    std::size_t sz1, // Size of the original array
    std::size_t sz2, // Size of the array of interpolated values
    double *z1, // Independent variable of original array
    double *z2, // Independent variable of target array
    double *f1, // Dependent variable of original array
    double *f2 // Dependent variable of target array (output)
);
void fill_holes(
    std::size_t sz,
    std::vector<bool> &mask,
    double *arr
);

} // namespace spex
