// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// SPEXone instrument spectral response function (ISRF). It is
// initialized using a NetCDF file containing the ISRF parameters.

#include <string>
#include <vector>

namespace spex {

class ISRF
{
private:
    std::vector<std::vector<double>> scale_a {};
    std::vector<std::vector<double>> shape_m {};
    std::vector<std::vector<double>> skew {};
    std::vector<std::vector<double>> radius_r {};

public:
    // Input file contains the 4 ISRF parameters for each L1B FOV and
    // column in a NetCDF format.
    ISRF(const std::string& filename);
    // The first two argument specify which ISRF to use - the FOV
    // (ACT) and detector column index. The last argument is the
    // detector column at which to evaluate the ISRF function.
    [[nodiscard]] auto eval(const int i_fov,
                            const int i_spec,
                            const double arg) const -> double;
};

} // namespace spex
