// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#include "l1a_batch.h"

#include "l1a.h"

#include <omp.h>
#include <spdlog/spdlog.h>

namespace spex {

static auto printInfo(const std::string_view prefix,
                      const ProcLevel proc_level,
                      const std::vector<L1A>& l1a_products) -> void
{
    if (omp_get_num_threads() > 1 || l1a_products.size() <= 1) {
        return;
    }
    if (l1a_products.front().isBackground()) {
        spdlog::info("{} background images", prefix);
    } else {
        spdlog::info("{} images", prefix);
    }
}

auto L1ABatch::calibrate(const SPEXSettings& settings,
                         const ProcLevel proc_level,
                         const CKD& ckd) -> void
{
    printInfo("Reading", proc_level, l1a_products);
    for (int i {}; i < static_cast<int>(l1a_products.size()); ++i) {
        l1a_products[i].readImage(ckd);
    }
    printInfo("Calibrating", proc_level, l1a_products);
#pragma omp parallel for
    for (auto& l1a : l1a_products) {
        l1a.calibrateImage(settings, proc_level, ckd);
    }
}

} // namespace spex
