// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// In general, L1A products are processed in batches. A batch may
// contain a single detector image, all images, or a subset of
// images. For instance, during CKD generation, depending on the
// process, it makes sense to partition L1A images into batches by
// viewport if the viewports can be processed independently. This
// class holds the L1A products and functions for reading and
// calibrating them.

#pragma once

#include "constants.h"
#include "spex_settings.h"

#include <vector>

namespace spex {

class CKD;
class L1A;

class L1ABatch
{
public:
    // All L1A products associated with this batch
    std::vector<L1A> l1a_products {};
    // Read and calibrate all L1A images in this batch
    auto calibrate(const SPEXSettings& settings,
                   const ProcLevel proc_level,
                   const CKD& ckd) -> void;
};

} // namespace spex
