// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// L1A container for calibration data. Each calibration measurement
// has an associated L1A product from which the images (frames) are
// read and then coadded. This is in contrast to flight data where all
// measurements are stored in a single file.

#pragma once

#include "l1a_manager.h"

namespace spex {

class L1ADiagnostic : public L1AManager
{
public:
    L1ADiagnostic(const bool type_background) : L1AManager { type_background }
    {}

private:
    auto getNumberOfImages(const SPEXSettings& /*settings*/,
                           const std::shared_ptr<L1AMetadata>& /*meta*/,
                           const size_t n_files) const -> int override;
    auto organize(const SPEXSettings& settings,
                  const ProcLevel proc_level,
                  std::vector<std::shared_ptr<L1AMetadata>>& meta)
      -> void override;
};

} // namespace spex
