// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Base class for sorting and and holding multiple L1A
// products. Inheriting classes are holding either calibration
// (diagnostic) or flight (science) data.

#pragma once

#include "l1a.h"

namespace spex {

class L1AManager
{
protected:
    L1AManager(const bool is_background) : is_background { is_background } {}

public:
    std::vector<L1A> l1a_products {};
    auto init(const SPEXSettings& settings,
              const ProcLevel proc_level,
              const CKD& ckd) -> void;
    auto initL1X(const SPEXSettings& settings,
                 const ProcLevel proc_level,
                 const CKD& ckd) -> void;
    virtual ~L1AManager() = default;

private:
    // If processing background images
    bool is_background {};
    virtual auto getNumberOfImages(const SPEXSettings& settings,
                                   const std::shared_ptr<L1AMetadata>& meta,
                                   const size_t n_files) const -> int = 0;
    // Perform some organization. Make sure each image has its shared
    // pointer to the right meta data object and assign correct frames
    // to each image. Here meta is passed as non-const because in case
    // some frames are discarded (diagnostic mode only) n_frames needs
    // to be modified.
    virtual auto organize(const SPEXSettings& settings,
                          const ProcLevel proc_level,
                          std::vector<std::shared_ptr<L1AMetadata>>& meta)
      -> void = 0;
};

} // namespace spex
