// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Linear algebra routines

#pragma once

#include "packed_matrix.h"

extern "C"
{
    auto dgttrf_(const int*, double*, double*, double*, double*, int*, int*)
      -> void;
    auto dgttrs_(const char*,
                 const int*,
                 const int*,
                 const double*,
                 const double*,
                 const double*,
                 const double*,
                 const int*,
                 double*,
                 const int*,
                 int*) -> void;
    auto dppsv_(const char*,
                const int*,
                const int*,
                double*,
                double*,
                const int*,
                int*) -> void;
    auto ddot_(const int*, const double*, const int*, const double*, const int*)
      -> double;
}

namespace spex {

// Compute the solution to a real system of linear equations A * X =
// B, where A is an N-by-N symmetric positive definite matrix stored
// in packed format and X and B are N-by-NRHS matrices.
auto dppsv(const std::vector<double>& data,
           const std::vector<double>& std_devs,
           std::vector<double>& B,
           std::vector<double>& state_vector) -> void;

// Same but the B matrix is stored in packed format
auto dppsv(const std::vector<double>& data,
           const std::vector<double>& std_devs,
           PackedMatrix& B,
           std::vector<double>& state_vector) -> void;

} // namespace spex
