// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#pragma once

#include "matrix.h"

#include <vector>

namespace spex {

class Lininv
{
public:
    size_t nstate {};
    size_t nmeas {};
    double *jac {};
    squareoption_t opt_s_y {};
    double *s_y {};
    std::vector<bool> excluded {};
    double *residual {};
    // Ouptut
    bool opt_res = true;
    bool opt_sx = false;
    bool opt_gain = false;
    std::vector<double> res {};
    std::vector<double> sx {};
    std::vector<double> gain {};

    int execute();
    ~Lininv();
};

// Out-of-class inversion routines, using the class internally
int linear_invert(
    size_t nstate, // Number of state parameters
    size_t nmeas, // Number of measurements, should be at least as high as the number of state parameters
    double *jac, // Jacobian matrix
    squareoption_t opt_s_y, // Type of error covariance matrix
    double *s_y, // Error covariance matrix
    std::vector<bool> *mask, // Pixel mask
    double *residual, // Measurement residual
    double *res, // Retrieved state
    double *sx=NULL // Error covariance on retrieved state
);

int linear_invert_gain(
    size_t nstate, // Number of state parameters
    size_t nmeas, // Number of measurements, should be at least as high as the number of state parameters
    double *jac, // Jacobian matrix
    squareoption_t opt_s_y, // Type of error covariance matrix
    double *s_y, // Error covariance matrix
    std::vector<bool> *mask, // Pixel mask
    double *gain // Gain matrix
);

// Linear inversion with hard constraints. Works the same as a regular linear
// inversion, but as a boundary condition, an ill-posed equation jac*res = residual
// is solved and the regular equation performs a least square fit using the
// remaining degrees of freedom in res.
int constrained_linear_invert(
    size_t nstate, // Number of state paramters
    size_t nmeas_soft, // Number of measurements for least-square fit
    size_t nmeas_hard, // Number of hard constraints
    double *jac_soft, // Jacobian matrix for least-square fit
    double *jac_hard, // Jacobian matrix for hard constraints
    squareoption_t opt_s_y_soft, // Definition option soft error covariance matrix
    double *s_y_soft, // Soft error covariance matrix
    std::vector<bool> *mask_soft, // Soft pixel mask
    double *residual_soft, // Residual measurements for least-square fit
    double *residual_hard, // Residual measurements for hard constraints
    double *res // The result vector
);

} // namespace spex
