// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Read in settings, check for errors and inconsistencies in settings,
// and run the SPEXone processor.
//
// When run without an argument print the default configuration and
// exit. Otherwise run the processor with settings from a YAML
// configuration file.

#include "driver.h"
#include "spex_settings.h"

#include <iostream>

auto main(int argc, char* argv[]) -> int
{
    if (argc == 1) {
        // When called without an argument print the default
        // configuration.
        std::cout << "%YAML 1.2\n---\n"
                  << spex::SPEXSettings {}.c_str() << '\n';
    } else {
        // NOLINTNEXTLINE(cppcoreguidelines-pro-bounds-pointer-arithmetic)
        spex::SPEXSettings settings { argv[1] };
        settings.init();
        spex::driver(settings);
    }
    return 0;
}
