// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#pragma once

#include "inversion.h"

namespace spex {

// This is an inversion routine that attempts to fit a set of Gaussians.
// This is used to recognize a number of peaks in a line.
class Multigaussfit : public Inversion
{
public:
    Multigaussfit(size_t a_ngauss, // Number of peaks (gausses)
                  size_t a_npoly, // Number of polynomial parameters for the background
                  size_t a_nmeas); // Number of elements in the measurement vector
    ~Multigaussfit();
    // TODO: Do this somehow a bit differently.
    // Maybe, the multigaussfit should have a private constructor and have
    // friended classes construct instances and access new_x or completely
    // keep track of this optimization.

public:
    bool new_x {};
    std::vector<double> mat {};

private:
    size_t ngauss {}; // Number of peaks
    size_t npoly {}; // Number of polynomial terms, one higher than the order
    std::vector<double> own_x {};
    double *x {};

    auto fwd(double *state, // State vector
             double *signal, // Modelled signal (outout)
             double *jacobian) -> void override; // Modelled jacobian (outout)

public:
    auto setX(double *a_x, // Independent parameter of the line
              bool copy=false) -> void; // Flag for copying the argument
};

} // namespace spex
