// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Class for generating the PRNU CKD. After correcting for
// nonlinearity the signal should be linear in the number of photons
// (intensity times exposure time) but the proportionality constant
// (sensitivity) differs per pixel. This is called pixel response
// non-uniformity (PRNU). To account for it we filter out the quick
// osciallations but keep the slow ones.

#pragma once

#include "processor.h"

namespace spex {

class PrnuGen : public Processor
{
public:
    PrnuGen(const SPEXSettings& settings, CKD& ckd)
      : Processor { settings, ProcLevel::prnu, ckd }
    {}
    ~PrnuGen() = default;

protected:
    auto processBatch(const int /*i_batch*/) -> void override;
};

} // namespace spex
