// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#include "read_solar_spectrum.h"

#include <netcdf>

auto readSolarSpectrum(const std::string& filename,
                       std::vector<double>& wavelengths,
                       std::vector<double>& spectrum) -> void
{
    const netCDF::NcFile nc_solar { filename, netCDF::NcFile::read };
    const auto n_wavelengths { nc_solar.getDim("wavelength").getSize() };
    wavelengths.resize(n_wavelengths);
    spectrum.resize(n_wavelengths);
    nc_solar.getVar("Vacuum Wavelength").getVar(wavelengths.data());
    nc_solar.getVar("SSI").getVar(spectrum.data());
    // From W m-2 nm-1 to W m-2 um-1
    for (double& val : spectrum) {
        val *= 1e3;
    }
}
