// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Class for generating the stray light CKD

#pragma once

#include "processor.h"

namespace spex {

class StrayGen : public Processor
{
private:
    // Pixel coordinates of the center of the kernel
    std::vector<std::array<double, 2>> centers {};

public:
    StrayGen(const SPEXSettings& settings, CKD& ckd)
      : Processor { settings, ProcLevel::stray, ckd }
    {}
    ~StrayGen() = default;

protected:
    // Create batches such that the polarization state, viewport, and
    // wavelength are constant within a batch.
    auto prepareBatches() -> void override;
    // Process background images in a way that's similar to
    // Processor::prepareBackground except that also make sure the
    // polarization state, wavelength, and exposure times match.
    auto prepareBackground(L1ADiagnostic& l1a_manager_bgr) -> void override;
    // Produce a stray light kernel from a set of L1A products
    // corresponding to the same polarization state, viewport, and
    // laser wavelength, but different ACT angles and exposure times.
    auto processBatch(const int i_batch) -> void override;
    auto processFinalize() -> void override;
};

} // namespace spex
