// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

#include "timer.h"

#include <omp.h>

namespace spex {

auto Timer::start() -> void
{
    // If we are in a nested parallel loop, identify the master thread
    // of the outer-most loop and allow only that one to update the
    // wall time stamp.
    if (omp_get_level() == 0 || omp_get_ancestor_thread_num(1) == 0) {
        wall_timestamp = std::chrono::high_resolution_clock::now();
    }
}

auto Timer::stop() -> void
{
    if (omp_get_level() == 0 || omp_get_ancestor_thread_num(1) == 0) {
        total_wall_time +=
          std::chrono::duration_cast<std::chrono::duration<double>>(
            std::chrono::high_resolution_clock::now() - wall_timestamp)
            .count();
        ++n_calls;
    }
}

[[nodiscard]] auto Timer::time() const -> double
{
    return total_wall_time;
}

[[nodiscard]] auto Timer::calls() const -> int
{
    return n_calls;
}

} // namespace spex
