// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Timer class for this project. It measures time for the OpenMP level
// 1 master thread. In the case of nested parallelism this typically
// corresponds to a loop over images (L1A products).

#pragma once

#include <chrono>

namespace gadfit {

class Timer;

} // namespace gadfit

namespace spex {

class Timer
{
private:
    std::chrono::time_point<std::chrono::high_resolution_clock> wall_timestamp;
    double total_wall_time {};
    int n_calls {};

public:
    Timer() = default;
    auto start() -> void;
    auto stop() -> void;
    // Return total wall time and the number of calls to start and stop
    [[nodiscard]] auto time() const -> double;
    [[nodiscard]] auto calls() const -> int;
};

} // namespace spex
