// This source code is licensed under the 3-clause BSD license found
// in the LICENSE file in the root directory of this project.

// Class for generating the spectral CKD

#pragma once

#include "bspline.h"
#include "splitgaussfit.h"
#include "processor.h"

namespace spex {

class WaveGen : public Processor
{
public:
    WaveGen(const SPEXSettings& settings, CKD& ckd)
      : Processor { settings, ProcLevel::wave, ckd }
    {}
    ~WaveGen() = default;

protected:
    auto processInit() -> void override;
    // Distribute L1A products into batches by viewports
    auto prepareBatches() -> void override;
    auto processBatch(const int i_vp) -> void override;

private:
    // Batch-to-viewport coupling
    std::vector<size_t> ivp_batch {};
    std::unique_ptr<Splitgaussfit> spl {};
    std::unique_ptr<Bspline> b {};
    // Matrix to evaluate the wavelength as function of spectral index
    std::vector<double> evalmat {};
};

} // namespace spex
