// Generation of dark CKD

#include "testing.h"

#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("Dark")
{
    // Main run
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/dark_gen.yaml" };
    settings.init();

    SECTION("Last exposure time fixed")
    {
        settings.main.ckd_file_out = settings.main.ckd_file_out + "_fixed.nc";
        spex::driver(settings);
        // Read the generated CKD
        settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
        spex::CKD ckd { spex::ProcLevel::n_levels, settings };
        // Read the reference CKD
        settings.main.ckd_file_in = TEST_DATA_DIR "/dark_gen/ckd_fixed_ref.nc";
        spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };
        CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
        CHECK_THAT(mae(ckd.dark.offset_short, ckd_ref.dark.offset_short),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.offset_long, ckd_ref.dark.offset_long),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.chi2, ckd_ref.dark.chi2),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.sigma_beta, ckd_ref.dark.sigma_beta),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.sigma_alpha, ckd_ref.dark.sigma_alpha),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(
          mae(ckd.dark.last_exposure_times, ckd_ref.dark.last_exposure_times),
          WithinAbs(0.0, 1e-11));
    }

    SECTION("Programmatic data selection")
    {
        settings.main.ckd_file_out = settings.main.ckd_file_out + "_varying.nc";
        settings.dark.last_exposure_time.reset();
        spex::driver(settings);
        settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
        spex::CKD ckd { spex::ProcLevel::n_levels, settings };
        settings.main.ckd_file_in =
          TEST_DATA_DIR "/dark_gen/ckd_varying_ref.nc";
        spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };
        CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
        CHECK_THAT(mae(ckd.dark.offset_short, ckd_ref.dark.offset_short),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.offset_long, ckd_ref.dark.offset_long),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.chi2, ckd_ref.dark.chi2),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.sigma_beta, ckd_ref.dark.sigma_beta),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(mae(ckd.dark.sigma_alpha, ckd_ref.dark.sigma_alpha),
                   WithinAbs(0.0, 1e-11));
        CHECK_THAT(
          mae(ckd.dark.last_exposure_times, ckd_ref.dark.last_exposure_times),
          WithinAbs(0.0, 1e-11));
    }
}
