// Generation of FOV CKD

#include "testing.h"

#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("FOV")
{
    // Main run
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/fov_gen.yaml" };
    settings.init();
    spex::driver(settings);

    // Read the generated CKD
    settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
    spex::CKD ckd { spex::ProcLevel::n_levels, settings };

    // Read the reference CKD
    settings.main.ckd_file_in = TEST_DATA_DIR "/fov_gen/ckd_ref.nc";
    spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };

    CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
    CHECK(std::ranges::equal(ckd.fov.n_act_vp, ckd_ref.fov.n_act_vp));
    CHECK(std::ranges::equal(ckd.fov.start_vp, ckd_ref.fov.start_vp));
    for (int i_pol {}; i_pol < spex::dims::pol; ++i_pol) {
        for (int i_act {}; i_act < ckd.fov.n_act; ++i_act) {
            CHECK_THAT(mae(ckd.fov.row_indices.at(i_pol).at(i_act),
                           ckd_ref.fov.row_indices.at(i_pol).at(i_act)),
                       WithinAbs(0.0, 1e-13));
            CHECK_THAT(mae(ckd.fov.peak_pos.at(i_pol).at(i_act),
                           ckd_ref.fov.peak_pos.at(i_pol).at(i_act)),
                       WithinAbs(0.0, 1e-13));
            CHECK_THAT(mae(ckd.fov.peak_pos_b.at(i_pol).at(i_act),
                           ckd_ref.fov.peak_pos_b.at(i_pol).at(i_act)),
                       WithinAbs(0.0, 1e-13));
        }
    }
}
