// L1X

#include "testing.h"

#include <omp.h>
#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/nc_object.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("L1X")
{
    omp_set_num_threads(1);
    // First generate the full L1B product with l1x_prnu.nc as an L1X
    // side product.
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/l1x.yaml" };
    settings.init();
    spex::driver(settings);

    // Next, using l1x_prnu.nc as input, generate a dummy L1B product,
    // l1b_after_prnu.nc, and an L1X product, l1x_fov.nc.
    settings.l1x_fov.at(static_cast<int>(spex::ProcLevel::l1b)) = "l1x_fov.nc";
    settings.l1x_prnu.at(static_cast<int>(spex::ProcLevel::l1b)) = "";
    settings.l1a_files.at(static_cast<int>(spex::ProcLevel::l1b))[0] =
      "l1x_prnu.nc";
    settings.l1b.outputfile = "l1b_after_prnu.nc";
    spex::driver(settings);

    // In the final run, use l1x_fov.nc as input and generate an L1B product
    settings.l1x_fov.at(static_cast<int>(spex::ProcLevel::l1b)) = "";
    settings.l1a_files.at(static_cast<int>(spex::ProcLevel::l1b))[0] =
      "l1x_fov.nc";
    settings.l1b.outputfile = "l1b_after_fov.nc";
    spex::driver(settings);

    spex::NcObject l1b { netCDF::NcFile::read,
                         CMAKE_CURRENT_BINARY_DIR "/l1b_after_fov.nc" };
    spex::NcObject l1b_ref { netCDF::NcFile::read,
                             TEST_DATA_DIR "/l1x/l1b_ref.nc" };
    const auto bins_along_track {
        l1b_ref.getDim("bins_along_track").getSize()
    };
    const auto spatial_samples_per_image {
        l1b_ref.getDim("spatial_samples_per_image").getSize()
    };
    const auto intensity_bands_per_view {
        l1b_ref.getDim("intensity_bands_per_view").getSize()
    };
    const auto polarization_bands_per_view {
        l1b_ref.getDim("polarization_bands_per_view").getSize()
    };
    std::vector<float> radiance(bins_along_track * spatial_samples_per_image
                                * intensity_bands_per_view);
    std::vector<float> radiance_ref(radiance.size());
    l1b.getGroup("OBSERVATION_DATA").getVar("I").getVar(radiance.data());
    l1b_ref.getGroup("OBSERVATION_DATA")
      .getVar("I")
      .getVar(radiance_ref.data());
    std::vector<float> radiance_noise(radiance.size());
    std::vector<float> radiance_noise_ref(radiance.size());
    l1b.getGroup("OBSERVATION_DATA")
      .getVar("I_noise")
      .getVar(radiance_noise.data());
    l1b_ref.getGroup("OBSERVATION_DATA")
      .getVar("I_noise")
      .getVar(radiance_noise_ref.data());
    std::vector<float> dolp(bins_along_track * spatial_samples_per_image
                            * polarization_bands_per_view);
    std::vector<float> dolp_ref(dolp.size());
    l1b.getGroup("OBSERVATION_DATA").getVar("DoLP").getVar(dolp.data());
    l1b_ref.getGroup("OBSERVATION_DATA").getVar("DoLP").getVar(dolp_ref.data());
    std::vector<float> dolp_noise(dolp.size());
    std::vector<float> dolp_noise_ref(dolp.size());
    l1b.getGroup("OBSERVATION_DATA")
      .getVar("DoLP_noise")
      .getVar(dolp_noise.data());
    l1b_ref.getGroup("OBSERVATION_DATA")
      .getVar("DoLP_noise")
      .getVar(dolp_noise_ref.data());
    CHECK_THAT(mae(radiance, radiance_ref), WithinAbs(0.0, 1e-8));
    CHECK_THAT(mae(radiance_noise, radiance_noise_ref), WithinAbs(0.0, 1e-8));
    CHECK_THAT(mae(dolp, dolp_ref), WithinAbs(0.0, 1e-8));
    CHECK_THAT(mae(dolp_noise, dolp_noise_ref), WithinAbs(0.0, 1e-8));
}
