// Generation of noise CKD

#include "testing.h"

#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("Noise")
{
    // Main run
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/noise_gen.yaml" };
    settings.init();
    spex::driver(settings);

    // Read the generated CKD
    settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
    spex::CKD ckd { spex::ProcLevel::n_levels, settings };

    // Read the reference CKD
    settings.main.ckd_file_in = TEST_DATA_DIR "/noise_gen/ckd_ref.nc";
    spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };

    CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
    CHECK_THAT(mae(ckd.noise.g, ckd_ref.noise.g), WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.noise.n, ckd_ref.noise.n), WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.noise.std_dev_g, ckd_ref.noise.std_dev_g),
               WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.noise.std_dev_n, ckd_ref.noise.std_dev_n),
               WithinAbs(0.0, 1e-9));
}
