// Generation of nonlinearity CKD

#include "testing.h"

#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("Nonlinearity")
{
    // Main run
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/nonlin_gen.yaml" };
    settings.init();

    spex::driver(settings);

    // Read the generated CKD
    settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
    spex::CKD ckd { spex::ProcLevel::n_levels, settings };

    // Read the reference CKD
    settings.main.ckd_file_in = TEST_DATA_DIR "/nonlin_gen/ckd_ref.nc";
    spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };

    CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
    CHECK(
      std::ranges::equal(ckd.nonlin.debug_pixels, ckd_ref.nonlin.debug_pixels));
    CHECK_THAT(mae(ckd.nonlin.f1, ckd_ref.nonlin.f1), WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.nonlin.f2, ckd_ref.nonlin.f2), WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.nonlin.m0, ckd_ref.nonlin.m0), WithinAbs(0.0, 1e-9));
    CHECK_THAT(std::abs(ckd.nonlin.m1 - ckd_ref.nonlin.m1),
               WithinAbs(0.0, 1e-9));
    CHECK_THAT(std::abs(ckd.nonlin.m2 - ckd_ref.nonlin.m2),
               WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.nonlin.S_max_data, ckd_ref.nonlin.S_max_data),
               WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.nonlin.S_max_fit, ckd_ref.nonlin.S_max_fit),
               WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.nonlin.I_c_med, ckd_ref.nonlin.I_c_med),
               WithinAbs(0.0, 1e-9));
    for (int i_r {}; i_r < ckd.nonlin_n_r; ++i_r) {
        CHECK_THAT(mae(ckd.nonlin.r[i_r], ckd_ref.nonlin.r[i_r]),
                   WithinAbs(0.0, 1e-9));
    }
    CHECK_THAT(mae(ckd.nonlin.c, ckd_ref.nonlin.c), WithinAbs(0.0, 1e-9));
    for (int i_pix {}; i_pix < static_cast<int>(ckd.nonlin.debug_pixels.size());
         ++i_pix) {
        CHECK_THAT(mae(ckd.nonlin.signal_rates[i_pix],
                       ckd_ref.nonlin.signal_rates[i_pix]),
                   WithinAbs(0.0, 1e-9));
        CHECK_THAT(mae(ckd.nonlin.reference_rates[i_pix],
                       ckd_ref.nonlin.reference_rates[i_pix]),
                   WithinAbs(0.0, 1e-9));
        CHECK_THAT(mae(ckd.nonlin.S_lin[i_pix], ckd_ref.nonlin.S_lin[i_pix]),
                   WithinAbs(0.0, 1e-9));
        CHECK_THAT(mae(ckd.nonlin.S_meas[i_pix], ckd_ref.nonlin.S_meas[i_pix]),
                   WithinAbs(0.0, 1e-9));
        CHECK_THAT(mae(ckd.nonlin.nonlinearity_fits[i_pix],
                       ckd_ref.nonlin.nonlinearity_fits[i_pix]),
                   WithinAbs(0.0, 1e-9));
    }
    CHECK_THAT(
      mae(ckd.nonlin.linear_spline.knots, ckd_ref.nonlin.linear_spline.knots),
      WithinAbs(0.0, 1e-9));
    CHECK_THAT(mae(ckd.nonlin.linear_spline.y, ckd_ref.nonlin.linear_spline.y),
               WithinAbs(0.0, 1e-9));
}
