// Generation of polarimetric CKD

#include "testing.h"

#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("Polarimetric")
{
    // Main run
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/pol_gen.yaml" };
    settings.init();

    SECTION("Model fit")
    {
        spex::driver(settings);
        // Read the generated and reference CKDs
        settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
        spex::CKD ckd { spex::ProcLevel::n_levels, settings };
        settings.main.ckd_file_in = TEST_DATA_DIR "/pol_gen/ckd_ref.nc";
        spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };
        CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
        for (int i {}; i < ckd.fov.n_act; ++i) {
            CHECK_THAT(mae(ckd.pol.m_q.at(i), ckd_ref.pol.m_q.at(i)),
                       WithinAbs(0.0, 1e-3));
            CHECK_THAT(mae(ckd.pol.m_u.at(i), ckd_ref.pol.m_u.at(i)),
                       WithinAbs(0.0, 1e-3));
            CHECK_THAT(mae(ckd.pol.m_t.at(i), ckd_ref.pol.m_t.at(i)),
                       WithinAbs(0.0, 1e-3));
            CHECK_THAT(mae(ckd.pol.eps.at(i), ckd_ref.pol.eps.at(i)),
                       WithinAbs(0.0, 1e-3));
            CHECK_THAT(mae(ckd.pol.delta.at(i), ckd_ref.pol.delta.at(i)),
                       WithinAbs(0.0, 1e-3));
        }
    }

    SECTION("Mueller matrix approach")
    {
        settings.main.ckd_file_out = settings.main.ckd_file_out + ".mueller";
        settings.pol.mueller.enabled = true;
        settings.pol.mueller.knots = 30;
        settings.pol.mueller.b_spline_order = 2;
        settings.pol.fit_window_min = 0.0;
        settings.pol.fit_window_max = 1e10;
        spex::driver(settings);
        // Read the generated and reference CKDs
        settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
        spex::CKD ckd { spex::ProcLevel::n_levels, settings };
        settings.main.ckd_file_in = TEST_DATA_DIR "/pol_gen/ckd_mueller_ref.nc";
        spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };
        CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
        for (int i {}; i < ckd.fov.n_act; ++i) {
            CHECK_THAT(mae(ckd.pol.m_q.at(i), ckd_ref.pol.m_q.at(i)),
                       WithinAbs(0.0, 1e-3));
            CHECK_THAT(mae(ckd.pol.m_u.at(i), ckd_ref.pol.m_u.at(i)),
                       WithinAbs(0.0, 1e-3));
            CHECK_THAT(mae(ckd.pol.eps.at(i), ckd_ref.pol.eps.at(i), 4, 5),
                       WithinAbs(0.0, 1e-3));
        }
    }
}
