// Generation of stray light CKD

#include "testing.h"

#include <spexone_cal/ckd.h>
#include <spexone_cal/driver.h>
#include <spexone_cal/spex_settings.h>

using Catch::Matchers::WithinAbs;

TEST_CASE("Stray light")
{
    // Main run
    spex::SPEXSettings settings { CMAKE_CURRENT_BINARY_DIR "/stray_gen.yaml" };
    settings.init();
    spex::driver(settings);

    // Read the generated CKD
    settings.main.ckd_file_in = settings.main.ckd_file_out.c_str();
    spex::CKD ckd { spex::ProcLevel::n_levels, settings };

    // Read the reference CKD
    settings.main.ckd_file_in = TEST_DATA_DIR "/stray_gen/ckd_ref.nc";
    spex::CKD ckd_ref { spex::ProcLevel::n_levels, settings };

    CHECK(std::ranges::equal(ckd.pixel_mask, ckd_ref.pixel_mask));
    CHECK(std::ranges::equal(ckd.stray.kernel_rows, ckd_ref.stray.kernel_rows));
    CHECK(std::ranges::equal(ckd.stray.kernel_cols, ckd_ref.stray.kernel_cols));
    CHECK(std::ranges::equal(ckd.stray.kernel_fft_sizes,
                             ckd_ref.stray.kernel_fft_sizes));
    CHECK(std::ranges::equal(ckd.stray.edges, ckd_ref.stray.edges));
    CHECK_THAT(mae(ckd.stray.eta, ckd_ref.stray.eta), WithinAbs(0.0, 1e-12));
    CHECK_THAT(mae(ckd.stray.all_kernels, ckd_ref.stray.all_kernels),
               WithinAbs(0.0, 1e-12));
    for (int i_kernel {}; i_kernel < ckd.stray.n_kernels; ++i_kernel) {
        // Real and imaginary parts of complex arrays are tested separately
        std::vector<double> fft(ckd.stray.kernels_fft.at(i_kernel).size());
        std::vector<double> fft_ref(fft.size());
        std::ranges::transform(ckd.stray.kernels_fft.at(i_kernel),
                               fft.begin(),
                               [](const auto x) { return x.real(); });
        std::ranges::transform(ckd_ref.stray.kernels_fft.at(i_kernel),
                               fft_ref.begin(),
                               [](const auto x) { return x.real(); });
        CHECK_THAT(mae(fft, fft_ref), WithinAbs(0.0, 1e-12));
        CHECK_THAT(mae(ckd.stray.weights.at(i_kernel),
                       ckd_ref.stray.weights.at(i_kernel)),
                   WithinAbs(0.0, 1e-12));
    }
}
