#pragma once

#include <catch2/catch_test_macros.hpp>
#include <catch2/matchers/catch_matchers_floating_point.hpp>
#include <numeric>

// Mean average difference (MAE) of two arrays divided by the sum of
// all elements in one of the arrays. This is the same as MAE per
// element normalized by the average value of the array. Also include
// options to skip elements from the beginning and/or end of the
// arrays (there is typically more noise at the detector edges).
template <typename T>
auto mae(T& arr1, T& arr2, const int skip_begin = 0, const int skip_end = 0)
  -> double
{
    const double sum {
        std::transform_reduce(
          arr1.begin() + skip_begin,
          arr1.begin() + arr1.size() - skip_end,
          arr2.begin() + skip_begin,
          0.0,
          std::plus<>(),
          [](const auto a, const auto b) { return std::abs(a - b); }),
    };
    const double arr1_sum { std::accumulate(arr1.cbegin(), arr1.cend(), 0.0) };
    if (std::abs(arr1_sum) > 1e-100) {
        return sum / std::accumulate(arr1.cbegin(), arr1.cend(), 0.0);
    } else {
        return sum;
    }
}
